--Test using these commands after spawning one:
-- c_select(c_findnext("fireplace_X"))
-- c_sel().components.burnable.fxchildren[1].Transform:SetPosition(0,0,0)

local function prePlaceFurnitureFn(inst)
    -- inst.Transform:SetTwoFaced()
    inst.Transform:SetRotation(-90)
end

local function placeTestWallFn(radius)
return function(inst, pt)
    inst.AnimState:SetOrientation(ANIM_ORIENTATION.RotatingBillboard)  
    inst.Transform:SetRotation(-90)

    local interiorSpawner = GetWorld().components.interiorspawner 
    if interiorSpawner.current_interior then

        local originpt = interiorSpawner:getSpawnOrigin()
        local width = interiorSpawner.current_interior.width
        local depth = interiorSpawner.current_interior.depth

        local dist = 2
		local backwallx = originpt.x - depth/2
        local backdiff =  pt.x < (backwallx + dist)
        local rightdiff = pt.z > (originpt.z + width/2 - dist)
        local leftdiff =  pt.z < (originpt.z - width/2 + dist)

        local canbuild = backdiff and not rightdiff and not leftdiff
		
		local newpt = {x = pt.x, y=0, z=pt.z}
		if backdiff or pt.x < backwallx then
			newpt.x = backwallx
		end
		
        inst.Transform:SetPosition(newpt.x,newpt.y,newpt.z)

        local ents = TheSim:FindEntities(newpt.x, newpt.y, newpt.z, radius, {"wallsection"})

		-- print("CANBUILD", backdiff, rightdiff, leftdiff, #ents)
		
        return #ents < 1 and canbuild

    end
    
    return false
end
end

local placeTestWallFn4 = placeTestWallFn(4)
local placeTestWallFn3 = placeTestWallFn(3)

local function smash(inst)
    if inst.components.lootdropper then
        local interiorSpawner = GetWorld().components.interiorspawner 
        if interiorSpawner.current_interior then
            local originpt = interiorSpawner:getSpawnOrigin()
            local x, y, z = inst.Transform:GetWorldPosition()
            local dropdir = Vector3(originpt.x - x, 0.0, originpt.z - z):GetNormalized()
            inst.components.lootdropper.dropdir = dropdir
            inst.components.lootdropper:DropLoot()
        end
    end
    SpawnAt("collapse_small", inst)
	SpawnAt("ash", inst)
    if inst.SoundEmitter then
        inst.SoundEmitter:PlaySound("dontstarve/common/destroy_stone")
    end
	
    inst:Remove()
end    

local function setPlayerUncraftable(inst)
    inst:AddComponent("lootdropper")
    -- inst.entity:AddSoundEmitter()
    inst:AddComponent("workable")
    inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
    inst.components.workable:SetWorkLeft(1)
    inst.components.workable:SetOnFinishCallback( smash )
end

local function onBuilt(inst)
    setPlayerUncraftable(inst)
    inst.onbuilt = true
	
    if inst:HasTag("wallsection") then
        local pt = inst:GetPosition()
        local ents = TheSim:FindEntities(pt.x, pt.y, pt.z, 1, {"wallsection"})
        for i,ent in ipairs(ents) do
            if ent ~= inst then
               smash(ent)
            end
        end
    end           
end

local function onignite(inst)
    if not inst.components.cooker and not inst.coldfire then
        inst:AddComponent("cooker")
    end
	if inst.firescale and inst.components.burnable.fxchildren[1] then
		inst.components.burnable.fxchildren[1].Transform:SetScale(inst.firescale,inst.firescale,inst.firescale)
	end
end

local function onextinguish(inst)
    if inst.components.cooker then
        inst:RemoveComponent("cooker")
    end
    if inst.components.fueled then
        inst.components.fueled:InitializeFuelLevel(0)
    end
end

local function onsave(inst, data)
	
    if inst:HasTag("playercrafted") then
        data.playercrafted = true
    end
	
    if inst.onbuilt then
        data.onbuilt = inst.onbuilt
    end
end

local function onload(inst, data)
	
    if data.playercrafted then
        inst:AddTag("playercrafted")
    end
        
    if data.onbuilt then
        setPlayerUncraftable(inst)
        inst.onbuilt = data.onbuilt
    end
end

local function fn(Sim)
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	
	inst.AnimState:PlayAnimation("north")
	
	inst.AnimState:SetOrientation(ANIM_ORIENTATION.RotatingBillboard)
	inst.Transform:SetRotation(-90)
	-- inst.AnimState:SetScale(1, 1, 1)
	
	-- inst.AnimState:SetLayer(LAYER_WORLD_BACKGROUND)
	-- inst.AnimState:SetSortOrder(background)   
	-- inst.setbackground = background
	
	inst:AddTag("wallsection")
    inst:AddTag("campfire")
	
	inst.OnSave = onsave 
	inst.OnLoad = onload
	
    -----------------------
	
    inst:AddComponent("burnable")
    --inst.components.burnable:SetFXLevel(2)
    inst.components.burnable:MakeNotWildfireStarter()
    inst:ListenForEvent("onextinguish", onextinguish)
    inst:ListenForEvent("onignite", onignite)
    
    -------------------------
	
    inst:AddComponent("fueled")
    inst.components.fueled.maxfuel = TUNING.FIREPIT_FUEL_MAX
    inst.components.fueled.accepting = true
    
    inst.components.fueled:SetSections(4)
    inst.components.fueled.bonusmult = TUNING.FIREPIT_BONUS_MULT
    inst.components.fueled.ontakefuelfn = function() inst.SoundEmitter:PlaySound("dontstarve/common/fireAddFuel") end
    
    inst.components.fueled:SetUpdateFn( function()
        if inst.components.burnable and inst.components.fueled then
            inst.components.burnable:SetFXLevel(inst.components.fueled:GetCurrentSection(), inst.components.fueled:GetSectionPercent())
        end
    end)
    
    inst.components.fueled:SetSectionCallback( function(section)
        if section == 0 then
            inst.components.burnable:Extinguish() 
        else
            if not inst.components.burnable:IsBurning() then
                inst.components.burnable:Ignite()
            end
            
            inst.components.burnable:SetFXLevel(section, inst.components.fueled:GetSectionPercent())
            
        end
    end)
        
    inst.components.fueled:InitializeFuelLevel(0)
    
    -----------------------------
	
    inst:AddComponent("inspectable")
	inst.components.inspectable.nameoverride = "firepit"
    inst.components.inspectable.getstatus = function(inst)
        local sec = inst.components.fueled:GetCurrentSection()
        if sec == 0 then 
            return "OUT"
        elseif sec <= 4 then
            local t = {"EMBERS","LOW","NORMAL","HIGH"}
            return t[sec]
        end
    end
	
    -----------------------
	
	inst:DoTaskInTime(0, function() 
			if inst:HasTag("playercrafted") then
				setPlayerUncraftable(inst)
			end
		end)
	
	inst:ListenForEvent( "onbuilt", onBuilt )
	
	return inst
end


local function fn_ant()
	local inst = fn()
	
	inst.AnimState:SetBank("ant_cave_door")
	inst.AnimState:SetBuild("ant_cave_door")
	
	inst.AnimState:SetScale(1, .7, 1)
	
    inst.components.burnable:AddBurnFX("campfirefire", Vector3(0,.6,0.3) )
	
	return inst
end

-- local function fn_bat()
	-- local inst = fn()
	
	-- inst.AnimState:SetBank("doorway_cave")
	-- inst.AnimState:SetBuild("bat_cave_door")
	-- -- inst.AnimState:PlayAnimation("night_loop")
	
	-- inst.AnimState:SetScale(.7, .7, 1)
	
    -- inst.components.burnable:AddBurnFX("campfirefire", Vector3(0,.25,0) )
	
	-- return inst
-- end

local function fn_ruins()
	local inst = fn()
	
	inst.AnimState:SetBank("fireplace_pig_ruins")
	inst.AnimState:SetBuild("fireplace_pig_ruins")
	
	inst.AnimState:SetMultColour(1, .8, .8, 1)
	
    inst.components.burnable:AddBurnFX("campfirefire", Vector3(0,.2,0) )
	
	return inst
end

local function fn_ruins_b()
	local inst = fn()
	
	inst.AnimState:SetBank("fireplace_pig_ruins")
	inst.AnimState:SetBuild("fireplace_pig_ruins")
	inst.AnimState:PlayAnimation("north_b")
	
	inst.AnimState:SetMultColour(1, .7, .9, 1)
	
    inst.components.fueled.secondaryfueltype = "CHEMICAL"
	
    inst.components.burnable:AddBurnFX("coldfirefire", Vector3(0,-.15,-.15) )
	inst.coldfire = true
	
	return inst
end

local function fn_anthead()
	local inst = fn()
	
	inst.AnimState:SetBank("entrance")
	inst.AnimState:SetBuild("ant_queen_entrance")
	inst.AnimState:PlayAnimation("idle")
	
	inst.AnimState:SetScale(.6, .6, .6)
	
    inst.components.burnable:AddBurnFX("campfirefire", Vector3(0,.25,0) )
	
	return inst
end

local function fn_anthead_b()
	local inst = fn()
	
	inst.AnimState:SetBank("entrance")
	inst.AnimState:SetBuild("fireplace_anthead_blue")
	inst.AnimState:PlayAnimation("idle")
	-- inst.AnimState:OverrideSymbol("entrance_01","fireplace_anthead_blue", "entrance_01")
	
	inst.AnimState:SetScale(.6, .6, .6)
	
    inst.components.burnable:AddBurnFX("coldfirefire", Vector3(0,-.2,-.2) )
	inst.coldfire = true
	
	return inst
end

local function fn_dragoon()
	local inst = fn()
	
	inst.AnimState:SetBank("dragoon_den")
	inst.AnimState:SetBuild("dragoon_den")
	inst.AnimState:PlayAnimation("idle")
	
	inst.AnimState:SetScale(.8, .9, .8)
	
    inst.components.burnable:AddBurnFX("campfirefire", Vector3(0,.1,0) )
	
	return inst
end

-- local function fn_sandcastle()
	-- local inst = fn()
	
	-- inst.AnimState:SetBank("sand_castle")
	-- inst.AnimState:SetBuild("sand_castle")
	-- inst.AnimState:PlayAnimation("full")
	
	-- inst.AnimState:SetScale(1, 1, 1)
	
    -- inst.components.burnable:AddBurnFX("campfirefire", Vector3(0,.2,0) )
	
	-- return inst
-- end

local function fn_monkeybarrel()
	local inst = fn()
	
	inst.AnimState:SetBank("barrel")
	inst.AnimState:SetBuild("monkey_barrel")
	inst.AnimState:PlayAnimation("idle")
	
	inst.AnimState:SetScale(1, 1, 1)
	
    inst.components.burnable:AddBurnFX("campfirefire", Vector3(0,.2,0) )
	
	return inst
end

local function fn_birdcage()
	local inst = fn()
	
	inst.AnimState:SetBank("fireplace_birdcage")
	inst.AnimState:SetBuild("fireplace_birdcage")
	inst.AnimState:PlayAnimation("fg")
	
	inst.decor = SpawnPrefab("fireplace_birdcage_bg")
	inst.decor.entity:SetParent( inst.entity )
	
	inst.AnimState:SetScale(1.2, 1.2, 1.2)
	
    inst.components.burnable:AddBurnFX("campfirefire", Vector3(0, 0.1, 0.13) )
	inst.firescale = 0.9
	inst.decor.Transform:SetPosition(0, 0, 0.1) --behind the fire
	
	return inst
end

local function fn_birdcage_bg()
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	
	inst:AddTag("DECOR")
	-- inst:AddTag("wallsection")
	inst.persists = false
	
	inst.AnimState:SetBank("fireplace_birdcage")
	inst.AnimState:SetBuild("fireplace_birdcage")
	inst.AnimState:PlayAnimation("bg")
	
	inst.AnimState:SetOrientation(ANIM_ORIENTATION.RotatingBillboard)
	-- inst.Transform:SetRotation(-90)
	
	inst.AnimState:SetScale(1.2, 1.2, 1.2)
	
	return inst
end

local function fn_adventure()
	local inst = fn()
	
	inst.AnimState:SetBank("portal_adventure")
	inst.AnimState:SetBuild("portal_adventure")
	inst.AnimState:PlayAnimation("idle_loop_on", true)
	
	inst.AnimState:SetScale(.9, .9, .9)
	
    inst.components.burnable:AddBurnFX("campfirefire", Vector3(0,1.4,0.5) )
	inst.firescale = 0.6
	
	return inst
end

local function fn_salad()
	local inst = fn()
	
	inst.AnimState:SetBank("fireplace_salad")
	inst.AnimState:SetBuild("fireplace_salad")
	inst.AnimState:PlayAnimation("north")
	
    inst.components.burnable:AddBurnFX("campfirefire", Vector3(0, 0.35, 0) )
	
	return inst
end

local function fn_victorian()
	local inst = fn()
	
	inst.AnimState:SetBank("fireplace_victorian")
	inst.AnimState:SetBuild("fireplace_victorian")
	inst.AnimState:PlayAnimation("north")
	
    inst.components.burnable:AddBurnFX("campfirefire", Vector3(0, 0.65, 0) )
	
	return inst
end

local function fn_gorge()
	local inst = fn()
	
	inst.AnimState:SetBank("fireplace_gorge")
	inst.AnimState:SetBuild("fireplace_gorge")
	inst.AnimState:PlayAnimation("north")
	
    inst.components.burnable:AddBurnFX("campfirefire", Vector3(0, .8, 0.4) )
	
	return inst
end

local assets = {
	Asset("ANIM", "anim/fireplace_pig_ruins.zip"),
	Asset("ANIM", "anim/pig_ruins_door_blue.zip"),
	Asset("ANIM", "anim/ant_cave_door.zip"),
    Asset("ANIM", "anim/ant_queen_entrance.zip"),
    Asset("ANIM", "anim/dragoon_den.zip"),
    -- Asset("ANIM", "anim/sand_castle.zip"),
    Asset("ANIM", "anim/monkey_barrel.zip"),
    Asset("ANIM", "anim/portal_adventure.zip"),
    Asset("ANIM", "anim/fireplace_anthead_blue.zip"),
    Asset("ANIM", "anim/fireplace_birdcage.zip"),
    Asset("ANIM", "anim/fireplace_salad.zip"),
    Asset("ANIM", "anim/fireplace_victorian.zip"),
    Asset("ANIM", "anim/fireplace_gorge.zip"),
}

local prefabs = {
    "campfirefire",
    "coldfirefire",
    "collapse_small",
}


return Prefab("deco/fireplace_ant_cave", fn_ant, assets, prefabs),
MakePlacer("fireplace_ant_cave_placer",
	"ant_cave_door", 
	"ant_cave_door", 
	"north", 
	nil, nil, nil, .9, nil, nil, nil, nil, nil, 
	placeTestWallFn3, nil, prePlaceFurnitureFn),
-- Prefab("deco/fireplace_bat_cave", fn_bat, assets, prefabs),
-- MakePlacer("fireplace_bat_cave_placer",
	-- "doorway_cave", 
	-- "bat_cave_door", 
	-- "north",  --night_loop
	-- nil, nil, nil, .9, nil, nil, nil, nil, nil, 
	-- placeTestWallFn3, nil, prePlaceFurnitureFn),
Prefab("deco/fireplace_pig_ruins", fn_ruins, assets, prefabs),
Prefab("deco/fireplace_pig_ruins_blue", fn_ruins_b, assets, prefabs),
MakePlacer("fireplace_pig_ruins_placer",
	"fireplace_pig_ruins", 
	"fireplace_pig_ruins", 
	"north", 
	nil, nil, nil, 1, nil, nil, nil, nil, nil, 
	placeTestWallFn4, nil, prePlaceFurnitureFn),
Prefab("deco/fireplace_ant_queen", fn_anthead, assets, prefabs),
Prefab("deco/fireplace_ant_queen_blue", fn_anthead_b, assets, prefabs),
MakePlacer("fireplace_ant_queen_placer",
	"entrance", 
	"ant_queen_entrance", 
	"idle", 
	nil, nil, nil, .78, nil, nil, nil, nil, nil, 
	placeTestWallFn4, nil, prePlaceFurnitureFn),
-- Prefab("deco/fireplace_sandcastle", fn_sandcastle, assets, prefabs),
-- MakePlacer("fireplace_sandcastle_placer",
	-- "sand_castle", 
	-- "sand_castle", 
	-- "full", 
	-- nil, nil, nil, 1, nil, nil, nil, nil, nil, 
	-- placeTestWallFn3, nil, prePlaceFurnitureFn),
Prefab("deco/fireplace_monkeybarrel", fn_monkeybarrel, assets, prefabs),
MakePlacer("fireplace_monkeybarrel_placer",
	"barrel", 
	"monkey_barrel", 
	"idle", 
	nil, nil, nil, 1, nil, nil, nil, nil, nil, 
	placeTestWallFn4, nil, prePlaceFurnitureFn),
Prefab("deco/fireplace_dragoon", fn_dragoon, assets, prefabs),
MakePlacer("fireplace_dragoon_placer",
	"dragoon_den", 
	"dragoon_den", 
	"idle",
	nil, nil, nil, .8, nil, nil, nil, nil, nil, 
	placeTestWallFn4, nil, prePlaceFurnitureFn),
Prefab("deco/fireplace_birdcage", fn_birdcage, assets, prefabs),
Prefab("deco/fireplace_birdcage_bg", fn_birdcage_bg, assets, prefabs),
MakePlacer("fireplace_birdcage_placer",
	"fireplace_birdcage", 
	"fireplace_birdcage", 
	"anim",
	nil, nil, nil, 1.1, nil, nil, nil, nil, nil, 
	placeTestWallFn3, nil, prePlaceFurnitureFn),
Prefab("deco/fireplace_adventure", fn_adventure, assets, prefabs),
MakePlacer("fireplace_adventure_placer",
	"portal_adventure", 
	"portal_adventure", 
	"idle_loop_on",
	nil, nil, nil, .95, nil, nil, nil, nil, nil, 
	placeTestWallFn4, nil, prePlaceFurnitureFn),
Prefab("deco/fireplace_salad", fn_salad, assets, prefabs),
MakePlacer("fireplace_salad_placer",
	"fireplace_salad", 
	"fireplace_salad", 
	"north",
	nil, nil, nil, 1, nil, nil, nil, nil, nil, 
	placeTestWallFn3, nil, prePlaceFurnitureFn),
Prefab("deco/fireplace_victorian", fn_victorian, assets, prefabs),
MakePlacer("fireplace_victorian_placer",
	"fireplace_victorian", 
	"fireplace_victorian", 
	"north",
	nil, nil, nil, 1, nil, nil, nil, nil, nil, 
	placeTestWallFn3, nil, prePlaceFurnitureFn),
Prefab("deco/fireplace_gorge", fn_gorge, assets, prefabs),
MakePlacer("fireplace_gorge_placer",
	"fireplace_gorge", 
	"fireplace_gorge", 
	"north",
	nil, nil, nil, 1, nil, nil, nil, nil, nil, 
	placeTestWallFn3, nil, prePlaceFurnitureFn)
	
	